package flow

import "time"

type Hooks []Hook

// Hook funcs to handle certain events on the flow
type Hook struct {
	Wait   func(ID string, triggerTime time.Time)
	Start  func(ID string, triggerTime time.Time)
	Finish func(ID string, triggerTime time.Time, res interface{})
	Error  func(ID string, triggerTime time.Time, err error)
	Any    func(name string, ID string, triggerTime time.Time, extra ...interface{})
}

// Trigger a hook
func (hooks Hooks) Trigger(name string, ID string, extra ...Data) {
	for _, h := range hooks {
		if h.Any != nil {
			h.Any(name, ID, time.Now(), extra...)
		}
		switch name {
		case "Wait":
			if h.Wait != nil {
				h.Wait(ID, time.Now())
			}
		case "Start":
			if h.Start != nil {
				h.Start(ID, time.Now())
			}
		case "Finish":
			if h.Finish != nil {
				h.Finish(ID, time.Now(), extra[0])
			}
		case "Error":
			if h.Error != nil {
				h.Error(ID, time.Now(), extra[0].(error))
			}
		}
	}
}

func (hooks Hooks) wait(ID string)             { hooks.Trigger("Wait", ID) }
func (hooks Hooks) start(ID string)            { hooks.Trigger("Start", ID) }
func (hooks Hooks) finish(ID string, res Data) { hooks.Trigger("Finish", ID, res) }
func (hooks Hooks) error(ID string, err error) { hooks.Trigger("Error", ID, err) }
